module Rakko

  # `Token` is a terminal symbol in grammer context.
  #
  # Its `value` attribute represents token's value and its syntactic category.
  # If `Token#value` is an instance of `Symbol`, its syntactic category is `Ident`.
  class Token
    attr_reader :lineno, :columnno, :value

    # @!attribute [r] lineno
    #   @return [Integer] The line position of the token (`0` base).
    #
    # @!attribute [r] columnno
    #   @return [Integer] The column position of the token (`0` base).

    # Creates a new Token instance.
    #
    # @param value token's value. If the value is `String`, it will be duplicated and frozen.
    # @param columnno [Integer] The column number of token position. number is `0` based.
    # @param lineno [Integer] The line number of token position. number is `0` based.
    def initialize(value, columnno: 0, lineno: 0)
      @columnno = columnno
      @lineno = lineno
      @value =
        case value
        when String
          value.dup.freeze
        else
          value
        end
    end

  end # Token

end
